"use strict";
/***********************************************************************
 * YoLink Platform Accessory class
 *
 * Copyright (c) 2022 David Kerr
 *
 * Based on https://github.com/homebridge/homebridge-plugin-template
 *
 * An instance of this class is created for each accessory the platform registers.
 *
 */
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.YoLinkPlatformAccessory = void 0;
const semaphore_promise_1 = __importDefault(require("semaphore-promise"));
const deviceHandlers_1 = require("./deviceHandlers");
const unknownDevice_1 = require("./unknownDevice");
class YoLinkPlatformAccessory {
    constructor(platform, accessory) {
        var _a;
        this.platform = platform;
        this.accessory = accessory;
        Error.stackTraceLimit = 100;
        const device = accessory.context.device;
        this.deviceId = device.deviceId;
        // Special handling if we have two devices attached to the one accessory
        // only known case right now is for binding a garage door sensor with controller
        this.deviceType = (accessory.context.device2) ? 'GarageDoorCombo' : device.type;
        this.initializeDeviceVars(platform, device);
        // Now initialize device, creating the homebridge services as required.
        // If device type exists in our list of supported services...
        if (deviceHandlers_1.initDeviceService[this.deviceType]) {
            // And it is not experimental, or we are allowing experimental...
            if ((!deviceHandlers_1.deviceFeatures[this.deviceType].experimental || device.config.enableExperimental)) {
                // Then set accessory information...
                this.infoService = this.accessory.getService(platform.Service.AccessoryInformation);
                this.infoService
                    .setCharacteristic(platform.Characteristic.Manufacturer, 'YoLink')
                    .setCharacteristic(platform.Characteristic.Name, device.name)
                    .setCharacteristic(platform.Characteristic.FirmwareRevision, String(device.config.version))
                    .setCharacteristic(platform.Characteristic.Model, String((_a = device.config.model) !== null && _a !== void 0 ? _a : 'n/a'))
                    // YoLink does not return device serial number in the API, use deviceId instead.
                    .setCharacteristic(platform.Characteristic.SerialNumber, device.deviceId);
                this.infoService
                    .getCharacteristic(platform.Characteristic.Identify).onSet(this.handleIdentifySet.bind(this, device));
                // Many YoLink devices are battery powered, so makes sense to include
                // battery level service.  YoLink reports 0..4, we will convert to 0,25,50,75,100 percent
                // deliberately using 'device.type' here.  If 'device2' has battery handle that in the
                // initDeviceService function.
                if (device.hasBattery) {
                    // We use a name here because an accessory might have two batteries (e.g. GarageDoorCombo)
                    device.batteryService = accessory.getService('Battery')
                        || accessory.addService(platform.Service.Battery, 'Battery', 'battery');
                    device.batteryService
                        .setCharacteristic(platform.Characteristic.Name, device.name)
                        .setCharacteristic(platform.Characteristic.ChargingState, platform.api.hap.Characteristic.ChargingState.NOT_CHARGEABLE)
                        .setCharacteristic(platform.Characteristic.BatteryLevel, 100);
                    device.batteryService
                        .getCharacteristic(platform.Characteristic.BatteryLevel).onGet(this.handleBatteryGet.bind(this, device));
                }
                // And finally call the device specific initialization...
                deviceHandlers_1.initDeviceService[this.deviceType].bind(this)();
            }
            else {
                platform.log.warn(`Experimental device ${device.deviceMsgName} skipped. Enable experimental devices in config.`);
            }
        }
        else {
            // We do not have support for this device yet.
            unknownDevice_1.initUnknownDevice.bind(this)();
        }
        return (this);
    }
    /*********************************************************************
     * checkDeviceState
     *
     */
    initializeDeviceVars(platform, device) {
        var _a, _b, _c;
        var _d, _e;
        device.data = {};
        device.deviceMsgName = `${device.name} (${device.deviceId})`;
        device.lastReportAtTime = 0;
        (_a = (_e = device.config).refreshAfter) !== null && _a !== void 0 ? _a : (_e.refreshAfter = (_b = (_d = platform.config).refreshAfter) !== null && _b !== void 0 ? _b : (_d.refreshAfter = 3600));
        device.config.enableExperimental = platform.makeBoolean(device.config.enableExperimental, platform.config.enableExperimental);
        device.config.temperature = platform.makeBoolean(device.config.temperature, platform.config.deviceTemperatures);
        device.hasBattery = deviceHandlers_1.deviceFeatures[device.type].hasBattery;
        // Set updateTime to now, which will ensure retrieving data from YoLink
        // on our first pass through.
        device.updateTime = Math.floor(new Date().getTime() / 1000);
        // set time of last log message to way back when...
        device.reportAtTime = new Date(0);
        // We need to serialize requests to YoLink API for each device.  Multiple threads
        // can request state updates for a device at the same time.  This would not be good,
        // so we need a semaphore to make sure we don't send a 2nd request to the same
        // device before prior one has completed.
        device.semaphore = new semaphore_promise_1.default();
        // GarageDoor specific...
        (_c = device.timeout) !== null && _c !== void 0 ? _c : (device.timeout = 45);
        // targetState used to track if garage door has been requested to open or close
        device.targetState = '';
    }
    /*********************************************************************
     * checkDeviceState
     * Updates device status object, sending a request to the YoLink API if we
     * have no data yet, or it has been a long time since the data was updated.
     *
     * Calls to this function should be serialized with deviceSemaphore to
     * prevent sending multiple requests for the same data to the server.
     */
    async checkDeviceState(platform, device) {
        var _a, _b;
        try {
            platform.verboseLog(`checkDeviceState for ${device.deviceMsgName} (refresh after ${device.config.refreshAfter} seconds)`);
            const timestamp = Math.floor(new Date().getTime() / 1000);
            if ((device.config.refreshAfter === 0)
                || ((device.config.refreshAfter > 0) && (timestamp >= device.updateTime))) {
                // If we have never retrieved data from the device, or data is older
                // than period we want to allow, then retrieve new data from the device.
                // Else return with data unchanged.
                device.budp = await platform.yolinkAPI.getDeviceState(platform, device);
                if (device.budp) {
                    device.data = device.budp.data;
                    device.updateTime = timestamp + device.config.refreshAfter;
                    // reportAtTime is the earlier of the time stamp on this message, or
                    // or the time reported in the message from YoLink. We use this to
                    // only log (in like mode), when we have an update.
                    const msgTime = new Date(parseInt(device.budp.msgid));
                    const repTime = new Date((_b = (_a = device.data) === null || _a === void 0 ? void 0 : _a.reportAt) !== null && _b !== void 0 ? _b : '9999-12-31');
                    device.reportAtTime = (msgTime < repTime) ? msgTime : repTime;
                    this.updateBatteryInfo.bind(this, device)();
                }
                else {
                    device.data = undefined;
                    platform.log.error(`checkDeviceState received no data for ${device.deviceMsgName}`);
                }
            }
        }
        catch (e) {
            const msg = (e instanceof Error) ? e.stack : e;
            platform.log.info('Error in checkDeviceState' + platform.reportError + msg);
        }
        return (device.data);
    }
    /***********************************************************************
     * logDeviceState
     *
     */
    logDeviceState(device, msg) {
        // reportAtTime is the earlier of the time stamp on this message, or
        // or the time reported in the message from YoLink. We use this to
        // only log (in like mode), when we have an update.
        if (device.lastReportAtTime < device.reportAtTime.getTime()) {
            device.lastReportAtTime = device.reportAtTime.getTime();
            this.platform.log.info(`At ${device.reportAtTime.toLocaleString()}: Device state for ${device.deviceMsgName} is: ${msg}`);
        }
        else {
            this.platform.liteLog(`At ${device.reportAtTime.toLocaleString()}: Device state for ${device.deviceMsgName} is: ${msg}`);
        }
    }
    /*********************************************************************
     * refreshDataTimer
     * We run a timer for each device to update cached data from the YoLink servers.
     * Timer is set such that it will fire when the updateTime arrives and (via the
     * provided callback) will call above checkDeviceState function at just the
     * right time to force call to yolinkAPI.getDeviceState.  All this to optimize
     * performance of user experience.
     */
    async refreshDataTimer(handleGet) {
        const platform = this.platform;
        const device = this.accessory.context.device;
        platform.verboseLog(`Data refresh timer for ${device.deviceMsgName} fired`);
        await handleGet.bind(this)();
        if (device.config.refreshAfter >= 60) {
            // We don't allow for updates any more frequently than once a minute.
            const nextUpdateIn = (device.updateTime) ? Math.max(60, device.updateTime - Math.floor(new Date().getTime() / 1000)) : 60;
            // If there was no device.updateTime then error occurred, so default to 60 seconds.
            platform.verboseLog(`Set data refresh timer for ${device.deviceMsgName} to run in ${nextUpdateIn} seconds`);
            setTimeout(() => {
                this.refreshDataTimer(handleGet);
            }, nextUpdateIn * 1000);
        }
    }
    /***********************************************************************
     * updateBatteryInfo
     *
     */
    updateBatteryInfo(device) {
        var _a, _b;
        const platform = this.platform;
        let batteryLevel = 100;
        try {
            if (device.hasBattery) {
                // Some devices wrap battery information under a 'state' object.
                // If nothing defined then assume 100%
                batteryLevel = ((_b = ((_a = device.data.battery) !== null && _a !== void 0 ? _a : device.data.state.battery)) !== null && _b !== void 0 ? _b : 4) * 25;
                const msg = `Battery level for ${device.deviceMsgName} is: ${batteryLevel}%`;
                if (batteryLevel <= 25) {
                    device.batteryService.updateCharacteristic(platform.Characteristic.StatusLowBattery, platform.api.hap.Characteristic.StatusLowBattery.BATTERY_LEVEL_LOW);
                    this.platform.log.warn(msg);
                }
                else {
                    device.batteryService.updateCharacteristic(platform.Characteristic.StatusLowBattery, platform.api.hap.Characteristic.StatusLowBattery.BATTERY_LEVEL_NORMAL);
                    this.platform.verboseLog(msg);
                }
            }
        }
        catch (e) {
            const msg = (e instanceof Error) ? e.stack : e;
            platform.log.error('Error in updateBatteryInfo' + platform.reportError + msg);
        }
        return (batteryLevel);
    }
    /*********************************************************************
     * handleBatteryGet
     *
     */
    async handleBatteryGet(device) {
        // const device = this.accessory.context.device;
        const platform = this.platform;
        // serialize access to device data.
        const releaseSemaphore = await device.semaphore.acquire();
        let rc = 100;
        try {
            if (await this.checkDeviceState(platform, device)) {
                rc = this.updateBatteryInfo.bind(this, device)();
            }
        }
        catch (e) {
            const msg = (e instanceof Error) ? e.stack : e;
            platform.log.error('Error in handleBatteryGet' + platform.reportError + msg);
        }
        finally {
            await releaseSemaphore();
        }
        return (rc);
    }
    /*********************************************************************
     * handleIdentifySet
     *
     */
    async handleIdentifySet(device, value) {
        const platform = this.platform;
        // serialize access to device data.
        const releaseSemaphore = await device.semaphore.acquire();
        try {
            platform.log.info(`YoLink Device: ${device.deviceMsgName} identify '${value}' (unsupported)`);
        }
        catch (e) {
            const msg = (e instanceof Error) ? e.stack : e;
            platform.log.error('Error in handleIdentitySet' + platform.reportError + msg);
        }
        finally {
            await releaseSemaphore();
        }
    }
    /*********************************************************************
     * mqttMessage
     *
     */
    async mqttMessage(message) {
        var _a, _b;
        const device = this.accessory.context.device;
        const platform = this.platform;
        try {
            if (device.data) {
                // reportAtTime is the earlier of the time stamp on this message, or
                // or the time reported in the message from YoLink. We use this to
                // only log (in like mode), when we have an update.
                const msgTime = new Date(parseInt(message.msgid));
                const repTime = new Date((_b = (_a = message.data) === null || _a === void 0 ? void 0 : _a.reportAt) !== null && _b !== void 0 ? _b : '9999-12-31');
                device.reportAtTime = (msgTime < repTime) ? msgTime : repTime;
                this.updateBatteryInfo.bind(this, device)();
                if (deviceHandlers_1.mqttHandler[this.deviceType]) {
                    deviceHandlers_1.mqttHandler[this.deviceType].bind(this)(message);
                }
                else {
                    unknownDevice_1.mqttUnknownDevice.bind(this)(message);
                }
            }
            else {
                platform.log.warn(`MQTT: ${message.event} for uninitialized device ${device.deviceMsgName}`
                    + platform.reportError + JSON.stringify(message));
            }
        }
        catch (e) {
            const msg = (e instanceof Error) ? e.stack : e;
            platform.log.error('Error in mqttMessage' + platform.reportError + msg);
        }
        return;
    }
}
exports.YoLinkPlatformAccessory = YoLinkPlatformAccessory;
//# sourceMappingURL=platformAccessory.js.map